@@ -15,6 +15,7 @@
 # under the License.
 
 import json
+from itertools import chain
 
 from django.template.loader import get_template
 from django.template import Context
@@ -41,7 +42,6 @@ class ExpressionWidget(forms.Widget):
         ('count', _('count')), ('avg', _('avg'))])
         comparators = [['>', '>'], ['>=', '>='], ['<', '<'], ['<=', '<=']]
 
-
         local_attrs = {'service': '', 'func': func, 'comparators': comparators}
         local_attrs.update(final_attrs)
         context = Context(local_attrs)
@@ -75,53 +75,20 @@ class NotificationCreateWidget(forms.Select):
         super(NotificationCreateWidget, self).__init__(*args, **kwargs)
 
     def render(self, name, value, attrs=None, choices=()):
-        output = '<table id="notification_table" ' + \
-            'class="table table-condensed">'
-        output += '<thead><tr><th>%s</th></tr></thead>' % \
-            unicode(_("Name"))
-        if value:
-            idx = 1
-            for notification in value:
-                output += '<tr><td>'
-                output += ('<select id="id_notifications_%d" ' +
-                           'name="notifications_%d"> ') % (idx, idx)
-                options = self.render_options(
-                    choices,
-                    [notification['id']])
-                if options:
-                    output += options
-                output += '</select>'
-                output += '<td><a href="" id="remove_notif_button">X</a></td>'
-                output += '</td></tr>'
-                idx += 1
-        else:
-            output += '<tr><td>'
-            output += '<select id="id_notifications_1" '
-            output += 'name="notifications_1"> '
-            options = self.render_options(choices, [value])
-            if options:
-                output += options
-            output += '</select>'
-            output += '<td><a href="" id="remove_notif_button">X</a></td>'
-            output += '</td></tr>'
-        output += '</table>'
-        label = unicode(_("+ Add more"))
-        output += '<a href="" id="add_notification_button">%s</a>' % (label)
-        return html.format_html(output)
+        final_attrs = self.build_attrs(attrs, name=name)
+        tpl = get_template(constants.TEMPLATE_PREFIX + 'notification_field.html')
+
+        selected = [item['id'] for item in value] if value else []
+        data = []
+        for id, label, type, address in chain(self.choices, choices):
+            data.append((id, label, type, address, id in selected))
+
+        local_attrs = {'data': json.dumps(data)}
+        local_attrs.update(final_attrs)
+        return tpl.render(Context(local_attrs))
 
     def value_from_datadict(self, data, files, name):
-        notifications = []
-        i = 0
-        while True:
-            i += 1
-            notification_id = "%s_%d" % (name, i)
-            if notification_id in data:
-                if len(data[notification_id]) > 0:
-                    notifications.append({"id":
-                                         data[notification_id]})
-            else:
-                break
-        return notifications
+        return [{"id": _id} for _id in data.getlist(name)]
 
 
 class BaseAlarmForm(forms.SelfHandlingForm):
@@ -136,27 +103,31 @@ class BaseAlarmForm(forms.SelfHandlingForm):
         if create:
             expressionWidget = ExpressionWidget(initial)
             notificationWidget = NotificationCreateWidget()
+            matchByAttr = None
         else:
             expressionWidget = textWidget
             notificationWidget = NotificationCreateWidget()
+            matchByAttr = {'readonly':'readonly'}
 
         self.fields['name'] = forms.CharField(label=_("Name"),
                                               required=required,
                                               max_length=250,
-                                              widget=textWidget)
+                                              widget=textWidget,
+                                              help_text=_("An unique name of the alarm."))
         self.fields['expression'] = forms.CharField(label=_("Expression"),
                                                     required=required,
-                                                    widget=expressionWidget)
-        apply_to = [['1', _('individually')],
-                    ['2', _('as a group')]]
-        self.fields['apply_to'] = forms.ChoiceField(label=_("Apply function to metrics"),
-                                                    choices=apply_to,
-                                                    required=False,
-                                                    widget=forms.RadioSelect(),
-                                                    initial="1")
+                                                    widget=expressionWidget,
+                                                    help_text=_("An alarm expression."))
+        self.fields['match_by'] = forms.CharField(label=_("Match by"),
+                                                  required=False,
+                                                  initial="url,hostname,component,service",
+                                                  widget=forms.TextInput(attrs=matchByAttr),
+                                                  help_text=_("The metric dimensions used "
+                                                              "to create unique alarms."))
         self.fields['description'] = forms.CharField(label=_("Description"),
                                                      required=False,
-                                                     widget=textWidget)
+                                                     widget=textWidget,
+                                                     help_text=_("A description of an alarm."))
         sev_choices = [("LOW", _("Low")),
                        ("MEDIUM", _("Medium")),
                        ("HIGH", _("High")),
@@ -164,7 +135,10 @@ class BaseAlarmForm(forms.SelfHandlingForm):
         self.fields['severity'] = forms.ChoiceField(label=_("Severity"),
                                                     choices=sev_choices,
                                                     widget=choiceWidget,
-                                                    required=False)
+                                                    required=False,
+                                                    help_text=_("Severity of an alarm. "
+                                                                "Must be either LOW, MEDIUM, HIGH "
+                                                                "or CRITICAL. Default is LOW."))
         self.fields['state'] = forms.CharField(label=_("State"),
                                                required=False,
                                                widget=textWidget)
@@ -176,7 +150,9 @@ class BaseAlarmForm(forms.SelfHandlingForm):
         self.fields['notifications'] = NotificationField(
             label=_("Notifications"),
             required=False,
-            widget=notificationWidget)
+            widget=notificationWidget,
+            help_text=_("Notification methods. "
+                        "Notifications can be sent when an alarm state transition occurs."))
 
     def set_notification_choices(self, request):
         try:
@@ -185,15 +161,11 @@ class BaseAlarmForm(forms.SelfHandlingForm):
             notifications = []
             exceptions.handle(request,
                               _('Unable to retrieve notifications: %s') % e)
-        notification_choices = [(notification['id'], notification['name'])
+        notification_choices = [(notification['id'],
+                                 notification['name'],
+                                 notification['type'],
+                                 notification['address'])
                                 for notification in notifications]
-        if notification_choices:
-            if len(notification_choices) > 1:
-                notification_choices.insert(
-                    0, ("", unicode(_("Select Notification"))))
-        else:
-            notification_choices.insert(
-                0, ("", unicode(_("No notifications available."))))
 
         self.fields['notifications'].choices = notification_choices
 
@@ -228,7 +200,7 @@ class CreateAlarmForm(BaseAlarmForm):
                 expression=data['expression'],
                 description=data['description'],
                 severity=data['severity'],
-                match_by=request.POST['match_by'].split(',') if request.POST['apply_to'] == '1' else None,
+                match_by=data['match_by'].split(',') if data['match_by'] else None,
                 alarm_actions=alarm_actions,
                 ok_actions=alarm_actions,
                 undetermined_actions=alarm_actions,
